package com.aldiferdiyan.xmpp.flutter_xmpp;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.Log;
import android.widget.Toast;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

public class FlutterXmppPlugin implements MethodCallHandler, EventChannel.StreamHandler {

  private static final String TAG ="flutter_xmpp";

  private static final String CHANNEL = "flutter_xmpp/method";
  private static final String CHANNEL_STREAM = "flutter_xmpp/stream";

  private static Activity activity;

  private String jid_user = "";
  private String password = "";
  private String host = "";
  private Integer port;
  private BroadcastReceiver mBroadcastReceiver;

  private FlutterXmppPlugin(Activity activity) {
    this.activity = activity;
  }

  public static void registerWith(Registrar registrar) {

    //method channel
    final MethodChannel method_channel = new MethodChannel(registrar.messenger(), CHANNEL);
    method_channel.setMethodCallHandler(new FlutterXmppPlugin(registrar.activity()));

    //event channel
    final EventChannel event_channel = new EventChannel(registrar.messenger(), CHANNEL_STREAM);
    event_channel.setStreamHandler(new FlutterXmppPlugin(registrar.activity()));

  }


  // ****************************************
  // stream
  @Override
  public void onListen(Object o, EventChannel.EventSink eventSink) {
    Log.w(TAG, "adding listener");
    mBroadcastReceiver = get_message(eventSink);
    IntentFilter filter = new IntentFilter(FlutterXmppConnectionService.RECEIVE_MESSAGE);
    activity.registerReceiver(mBroadcastReceiver,filter);
  }

  @Override
  public void onCancel(Object o) {
    Log.w(TAG, "cancelling listener");
    activity.unregisterReceiver(mBroadcastReceiver);
    mBroadcastReceiver = null;
  }


  private static BroadcastReceiver get_message(final EventChannel.EventSink events) {
    return new BroadcastReceiver() {
      @Override
      public void onReceive(Context context, Intent intent) {
        String action = intent.getAction();
        switch (action) {
          case FlutterXmppConnectionService.RECEIVE_MESSAGE:
            String from = intent.getStringExtra(FlutterXmppConnectionService.BUNDLE_FROM_JID);
            String body = intent.getStringExtra(FlutterXmppConnectionService.BUNDLE_MESSAGE_BODY);
            Log.d(TAG,"msg : " + from + " : " + body);

            Map<String, Object> build = new HashMap<>();
            build.put("from", from);
            build.put("body", body);
            events.success(build);

        }
      }
    };
  }


  // ****************************************
  // method call
  @Override
  public void onMethodCall(MethodCall call, Result result) {

    // login
    if (call.method.equals("login")) {
      if (!call.hasArgument("user_jid") || !call.hasArgument("password") || !call.hasArgument("host") || !call.hasArgument("port")) {
        result.error("MISSING", "Missing argument jid_user/password/host/port.", null);
      }
      jid_user = call.argument("user_jid");
      password = call.argument("password");
      host = call.argument("host");
      port = call.argument("port");

      login();
      result.success("SUCCESS");


      // send message
    }else if(call.method.equals("send_message")){
      if (!call.hasArgument("to_jid") || !call.hasArgument("body")) {
        result.error("MISSING", "Missing argument to_jid / body.", null);
      }

      String to_jid = call.argument("to_jid");
      String body = call.argument("body");
      send_message(body,to_jid);

      result.success("SUCCESS");

      // logout
    }else if(call.method.equals("logout")){

     logout();

      result.success("SUCCESS");
    } else {
      result.notImplemented();
    }
  }

  // login
  private void login(){
    Intent i = new Intent(activity,FlutterXmppConnectionService.class);
    i.putExtra("jid_user", jid_user);
    i.putExtra("password", password);
    i.putExtra("host", host);
    i.putExtra("port", port);
    activity.startService(i);
  }

  private void logout(){
    Intent i1 = new Intent(activity,FlutterXmppConnectionService.class);
    activity.stopService(i1);
  }

  // send message to JID
  private void send_message(String msg,String jid_user){
    if (FlutterXmppConnectionService.getState().equals(FlutterXmppConnection.ConnectionState.CONNECTED)) {
      Log.d(TAG, "The client is connected to the server,Sending Message");
      Intent intent = new Intent(FlutterXmppConnectionService.SEND_MESSAGE);
      intent.putExtra(FlutterXmppConnectionService.BUNDLE_MESSAGE_BODY, msg);
      intent.putExtra(FlutterXmppConnectionService.BUNDLE_TO, jid_user);
      activity.sendBroadcast(intent);
    } else {
      Toast.makeText(activity, "Client not connected to server ,Message not sent!", Toast.LENGTH_LONG).show();
    }
  }


  private void set_presence(){

  }

  private void get_presence(){

  }


}
